/**************************************************************************
 *
 * frenchstem.c -- a simple french stemmer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: frenchstem.c 29 1998-11-24 06:37:53Z rjmcnab $
 *
 **************************************************************************/

#include "frenchstem.h"



/* =========================================================================
 * Function: frenchstem
 * Description: a simple french stemmer
 * Input: a word string with the length in the first byte
 * Output: the stemmed word
 * ========================================================================= */

void frenchstem (unsigned char *word) {      
  unsigned char *wordstart = word + 1; /* points to first letter of word */
  int last = (int)word[0]-1;           /* last points to the last character */

  if (last > 4) {
    if (wordstart[last]=='x') {
      if (wordstart[last-1]=='u' && wordstart[last-2]=='a') {
	wordstart[last-1]='l';
      }
      wordstart[last]='\0';
      last--;

    } else {
      if (wordstart[last]=='s' && last>=0) 
	{wordstart[last]='\0'; last--;}
      if (wordstart[last]=='r' && last>=0)
	{wordstart[last]='\0'; last--;}
      if (wordstart[last]=='e' && last>=0)
	{wordstart[last]='\0'; last--;}
      if (wordstart[last]=='' && last>=0) /* letter with accent e + ' */
	{wordstart[last]='\0'; last--;}
      if (wordstart[last]==wordstart[last-1] && last>=1)
	{wordstart[last]='\0'; last--;}
    }  /* end else */

    word[0] = (unsigned char)(last+1);
  } /* end if (len > 4) */
}
