/**************************************************************************
 *
 * stemmer.c -- The stemmer/case folder
 * Copyright (C) 1994  Neil Sharman
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: stemmer.c 13 1998-11-17 09:36:00Z rjmcnab $
 *
 **************************************************************************/

#include "sysfuncs.h"
#include "stemmer.h"

/*
   $Log$
   Revision 1.1  1998/11/17 09:35:42  rjmcnab
   *** empty log message ***

   * Revision 1.3  1994/10/20  03:57:05  tes
   * I have rewritten the boolean query optimiser and abstracted out the
   * components of the boolean query.
   *
   * Revision 1.2  1994/09/20  04:42:10  tes
   * For version 1.1
   *
 */

static char *RCSID = "$Id: stemmer.c 13 1998-11-17 09:36:00Z rjmcnab $";

/* [RJM 03/98: Extended ascii] */
u_char exttolower (u_char c) {
  if ((c >= 65 && c <= 90) ||
      (c >= 192 && c <= 214) ||
      (c >= 216 && c <= 222)) {
    return (c+32);
  }
  return c;
}

/*
 * Method 0 - Do not stem or case fold.
 * Method 1 - Case fold.
 * Method 2 - Stem.
 * Method 3 - Case fold and stem.
 */
void 
stemmer (int method, u_char * word)
{
  if (method & 1)
    {
      int len = *word;
      u_char *w = word + 1;

      for (; len > 0; len--, w++)
	*w = exttolower (*w);
    }
  if (method & 2) {
#ifdef FRENCHSTEMMER
    frenchstem (word);
#else
    stem (word);
#endif

  }
}
