#ifndef GDBMCLASS_H
#define GDBMCLASS_H

#ifdef __WIN32__
#include "autoconf.h"
#include "systems.h"
#include "gdbmconst.h"
#include "gdbm.h"

#else
#include <gdbm.h>
#endif

#include "locateinfo.h"


class gdbm_info 
{
public:
  int docnum;
  int d;
  text_t t;
  text_t p;
  text_t x;
  text_t c;
  text_t j;
  text_t o;
  text_t a;

  void clear ();
};


class gdbmclass 
{
public:
  gdbmclass() {gdbm_file = NULL; logout = &cerr;};
  ~gdbmclass() {};
  
  // returns 0 on success, -1 on failure
  int getinfo (text_t key, text_t collection, gdbm_info &info);
  void setgdbmhome (const text_t &thegdbmhome);
  void setlogout (ostream *thelogout) {logout = thelogout;}
  int exists (text_t key, text_t collection);

protected:
  text_t gdbmhome; // top level directory for gdbm files
  GDBM_FILE gdbm_file;
  text_t gdbm_open_name;
  ostream *logout;
  
  int open_database (const text_t &collection);
  void close_database ();
  
  int get_infoline (char *str, int len, int pos, char *key, char *value);
};


// a few useful functions

int is_top_level (const text_t &targetdoc);
int is_book (const text_t &targetdoc);
void get_book_top (const text_t &targetdoc, text_t &book_top);
void get_book (const text_t &targetdoc, text_t &book);
void get_parent_section (text_t &section);
void get_parent_section (text_t &parentsection, text_t &childsection);
int count_dots (text_t::const_iterator first, text_t::const_iterator last);
int count_dots (const text_t &targetdoc);
int is_section_top (const text_t &targetdoc);
void remove_tags (text_t &text);

void seperate_parts (const text_t &targetdoc, gdbmclass &gdbm, const text_t &collection, 
		     text_t &classification, text_t &booksection);
void split_targetdoc (const text_t &targetdoc, text_t &classification, 
		     text_t &booksection);
void splitstring (const text_t &string, vector<text_t> &array);
void get_parents (const text_t &targetdoc, vector<text_t> &parents);
void get_siblings (const text_t &classification, const text_t &booksection, 
		   gdbmclass &gdbm, const text_t &collection, 
		   vector<text_t> &siblings);
int are_same_chapter(text_t section1, text_t section2);
void get_first_section(const text_t &instring, text_t &returnstring);
int is_number (text_t &text);
text_t get_section_str(const text_t &str);
void alphabetize_string_english (text_t &text);
void alphabetize_string_name (text_t &text);
char ** string_add (char **array, int *len, char *str);
void string_sort (char **array, int len);
static int compare_str (const void *e1, const void *e2);
void string_free(char **array, int len);
#endif
